using System;
using System.IO;
using System.Collections;
using Views;

class PublicHolidays {

  /* The Public Holidays Program        Bishop & Horspool    Feb 2002
   * ===========================
   * Maintains a list of public holiday dates and enables queries on
   * dates. Illustrates collections and string splitters.
   * GUI version with a Date class.
   */

  SortedList H = new SortedList(20);

  Views.Form gui;

  string spec = @"<Form Text = Holidays>
    <vertical>
      <Label Text='Public Holiday Chooser' Font=sfbf16 ForeColor=red 
             Height=40 halign=center Width=350/>
      <horizontal>
        <Label   Text = 'Read the data from' Width=100/>
        <TextBox Name = filename text=holidays.dat Width = 80/>
        <Button  Name = 'read in'/>
      </horizontal>
      <horizontal>
        <vertical>
          <Label Text='List of holidays'/>
          <Listbox Name=list/>
        </vertical>
        <vertical>
          <Label Text='Type and Find or Choose and Select'
             Width=100 Height=50/>
          <Label Text='Date'/>
             <TextBox Name=date Text='25 December' 
             Width = 100/>
          <Button  Name=find Text=Find/>
          <space     height=20/>
          <Label   Text = 'Holiday'/>
          <TextBox Name = holiday Width = 150/>
          <Button  Name = select Text = Select/>
        </vertical>
      </horizontal>
    </vertical>
  </Form>";  
  
  void Go() {
    try {
      gui = new Views.Form(spec);
    } catch( Exception e ) {
      Console.WriteLine(e.Message);
      return;
    } 
    HandleQueries();
  }

  void HandleQueries() {
    while (gui != null) {
      string b = gui.GetControl();
      switch(b) {
        case "read in":
          ReadData(gui.GetText("filename"));
          break;
        case "find":
          HandleSelection (gui.GetText("date"));
          break;
        case "select":
          HandleSelection (gui.GetText("list"));
          break;
        default:
          break;
      } 
    } 
    gui.CloseGUI();
  }
  bool ReadData(string filename) {
    StreamReader fin = null;
    try {
      fin = File.OpenText(filename);
    } 
    catch (FileNotFoundException e) {
      gui.PutText("List", e.Message);
      return false;
    } 
    gui.PutText("list","");
    gui.PutText("list","Holidays from "+filename);

    string line;
    string[] elements;
    while ((line = fin.ReadLine()) != null) {
      elements = line.Split(' ');
      string date = elements[0] + " "+ elements[1];
      String name = "";
      // Join the rest of the tokens for
      // the holiday's name
      for (int i = 2; i<elements.Length; i++)
        name += ' '+elements[i];
      gui.PutText("list",date.ToString());
      H[date] = name;
    }
    return true;
  } 

  void HandleSelection(string date) {
    String text;
    text = (string) H[date];
    if (text == null)
      text = "Not a public holiday";
    gui.PutText("holiday", text);
    gui.PutText("date", date.ToString());
  }

  static void Main() {
    new PublicHolidays().Go();
  }
}
