using System;
using System.Collections;
using System.IO;
class StaffTraining {

  /// <summary>
  /// The Staff Training Courses Program   Bishop & Horspool Sept 2002
  /// ==================================
  /// Creates lists of employees required to attend courses according 
  /// to their rank in the organization.
  /// Illustrates use of collections and enumerated types.
  /// </summary>

  SortedList courses = new SortedList();
  SortedList employees = new SortedList();

  void Go() {
    ReadCourses("course.dat");
    ReadEmployees("employee.dat");
    OrganizeData();
    PrintResults();
  }

  void ReadCourses(string filename) {
    StreamReader fin = null;
    try {
      fin = new StreamReader(filename);
    } catch (IOException ioe) {
      Console.WriteLine(ioe.Message);
      return;
    }

    string line = fin.ReadLine().Trim();
    while (line != null) {
      if (line == "")
        continue;   // skip any blank lines
      string[] values = line.Split(' ');
      Employee.Ranks[] suggestions = 
                       new Employee.Ranks[values.Length - 1];
      for (int i = 1; i < values.Length; i++)
        suggestions[i - 1] = (Employee.Ranks) Enum.Parse(
            typeof(Employee.Ranks),values[i].Trim());
      Course c = new Course(values[0], suggestions);
      courses[c.Name] = c;
      line = fin.ReadLine();
    }
  }

  void ReadEmployees(string filename) {
    StreamReader fin = null;
    try {
      fin = new StreamReader(filename);
    } catch (IOException ioe) {
      Console.WriteLine(ioe.Message);
      return;
    }

    string line = fin.ReadLine();
    while (line != null) {
      if (line == "")
        continue;
      string[] values = line.Split(' ');
      Employee e = new Employee(values);
      employees[e.EmployeeNumber]=e;
      line = fin.ReadLine();
    }
  }

  void OrganizeData() {
    foreach(string courseName in courses.Keys) {
      Course c = (Course)courses[courseName];
      foreach(Employee.Ranks i in c.Suggested) {
        foreach(int employeeNumber in employees.Keys) {
          Employee e = (Employee)employees[employeeNumber];
          if (e.Rank == i)
            c.addAttendant(e);
        }
      }
    }
  }

  void PrintResults() {
    Console.WriteLine("*** Training Schedules ***");
    Console.WriteLine("The course lists");
    Console.WriteLine("================");
    foreach(string courseName in courses.Keys) {
      Course c = (Course)courses[courseName];
      Console.WriteLine(c.Name);
      if (c.Attendees != null) {
        Console.WriteLine("To be attended by:");
        foreach(int employeeNumber in c.Attendees.Keys) {
          Employee e = (Employee)c.Attendees[employeeNumber];
          Console.WriteLine("\t" + e);
        }
      }
      Console.WriteLine();
    }
    Console.WriteLine("The schedules");
    Console.WriteLine("=============");
    foreach(int employeeNumber in employees.Keys) {
      Employee e = (Employee)employees[employeeNumber];
      Console.WriteLine(e);
      Console.WriteLine("Is to attend:");
      foreach(string courseName in courses.Keys) {
        Course c = (Course)courses[courseName];
        if (c.Attendees != 
            null && c.Attendees.ContainsKey(employeeNumber))
          Console.WriteLine("\t" + courseName);
      }
      Console.WriteLine();
    }
  }

  static void Main(string[] args) {
    new StaffTraining().Go();
  }
}

class Course {
  string name;
  Employee.Ranks[] suggested;
  SortedList attendees;

  public string Name {
    get {return name;}
    set {name = value;}
  }
  public Employee.Ranks[] Suggested {
    get {return suggested;}
  }
  public SortedList Attendees {
    get {return attendees;}
  }

  public Course(string n, Employee.Ranks[] s) {
    name = n;
    suggested = s;
    attendees = null;
  }

  public void addAttendant(Employee o) {
    if (attendees == null)
      attendees = new SortedList();
    attendees[o.EmployeeNumber] = o;
  }
}

struct Employee {
  public enum Ranks {director, manager, researcher, 
                     technician, support, trainee}
  string title;
  string initials;
  string surname;
  int extension;
  Ranks rank;
  int employeeNumber;
  public Employee(string[] values) {
    title = values[1];
    initials = values[2];
    surname = values[3];
    extension = int.Parse(values[4]);
    rank = (Ranks) Enum.Parse(typeof(Ranks),values[5]);
    employeeNumber = int.Parse(values[0]);
  }

  override public string ToString() {
    return employeeNumber + " " + title + " " + initials + 
           " " + surname
      + " (" + rank.ToString()+")";
  }

  public int EmployeeNumber {
    get {return employeeNumber;}
  }

  public Ranks Rank {
    get {return rank;}
  }
}
