using System;
using System.IO;

class WeatherStats {

  ///<summary>
  /// The Weather program        Bishop & Horspool     2003 
  ///  ===================      Java by J M Bishop    Jan 1997
  ///                           updated June 2000
  ///
  ///  Calculates mean and standard deviation of rainfall
  ///  for each month over the number of years provided.
  ///  Illustrates handling of matrices and passing columns
  ///  as parameters.
  ///
  ///  The data must be in a file in the form:
  ///  year followed by the 12 rainfall figures for
  ///  the months of that year.
  /// </summary>

  int startYear, endYear; // range from 1950 upwards
  // all arrays intended to be indexed 1..12
  double[][] rainTable;
  double[] averagetable = new double[13];
  double[] stddevTable  = new double[13];

  void Go() {
    ReadIn();
    ShowResults();
  }

  void ReadIn() {
    StreamReader fin = new StreamReader("rain.dat");
    int yearIndex = 0;  // e.g. 0

    // The actual years are read in and might not be sorted
    // or contiguous. The yearIndex starts at 0 and is
    // used to store the data in an orderly manner.
    startYear = int.Parse(fin.ReadLine());
    endYear = int.Parse(fin.ReadLine());
    int nYears = endYear-startYear+1;
    rainTable  = new double[13][];
    for (int m=1; m<=12; m++)
      rainTable[m] = new double[nYears];

    string line;
    int year;
    while (true) {
        line = fin.ReadLine();
        if (line==null) break;
        string[] elements = line.Split(' ');
        year = int.Parse(elements[0]);
        Console.Write(year+" ");

        int i=1;
        for (int m = 1; m<=12; m++) {
          while (elements[i].CompareTo("")==0) 
            i++;
          rainTable[m][yearIndex] = double.Parse(elements[i]);
          Console.Write("{0,6:f1}", rainTable[m][yearIndex]);
          i++;
        } 
        Console.WriteLine();
        yearIndex++;
    }
    Console.WriteLine("Data read for "+startYear+" to "+
            endYear+"\n\n");
  }

  void ShowResults() {
      Console.WriteLine("Rainfall statistics for " +
          startYear + " to " + endYear);
      Console.WriteLine("====================================\n");
      Console.WriteLine("Month        Mean     Std Deviation");
    
      for (int m =1; m<=12; m++) {
        averagetable[m] = Stats.mean(rainTable[m]);
        stddevTable[m] = Stats.stddev(rainTable[m], averagetable[m]);
        Console.WriteLine("{0,-10}{1,8:f2}{2,14:f4}",
           new DateTime(2003,m,1).ToString("MMMM"), 
           averagetable[m], stddevTable[m]);
      }
  }

  static void Main() {
    new WeatherStats().Go();
  }
}